//    IPS Java Speech Database
//    (c) Copyright 2011
//    Institute of Phonetics and Speech Processing,
//    Ludwig-Maximilians-University, Munich, Germany
//
//
//    This file is part of IPS Java Speech Database
//
//
//    IPS Java Speech Database is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General Public License as published by
//    the Free Software Foundation, version 3 of the License.
//
//    IPS Java Speech Database is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with IPS Java Speech Database.  If not, see <http://www.gnu.org/licenses/>.

package ipsk.db.speech;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;

@Entity
@Table(name = "annotation_session")
public class AnnotationSessionDb extends AnnotationSession {

	/** default constructor */
	public AnnotationSessionDb() {
		super();
	}

	/** minimal constructor */
	public AnnotationSessionDb(int annotationSessionId) {
		super(annotationSessionId);
	}

	public AnnotationSessionDb(AnnotationSession source) {
		super(source);
	}

	public String toString() {
		String name = "ipsk.db.speech.AnnotationSessionDb";
		if (annotationSessionId == 0) {
			name = name.concat("-");
		} else {
			name = name.concat(Integer.toString(annotationSessionId));
		}
		name = name.concat("[" + type + "]:" + name + ")");
		return name;
	}

	public void merge(AnnotationSession changedProperties) {
		if (this.annotationSessionId != 0
				&& changedProperties.annotationSessionId != this.annotationSessionId) {
			throw new IllegalArgumentException(
					"Sessions do not have the same Id!");
		}
		super.merge(changedProperties);
	}

	@Id
	@Column(name = "annotation_session_id", unique = true, nullable = false)
	@GeneratedValue(generator = "id_gen")
	public int getAnnotationSessionId() {
		return annotationSessionId;
	}

	public void setAnnotationSessionId(int annotationSessionId) {
		this.annotationSessionId = annotationSessionId;
	}

	@Column(name = "annotation_session_name", length = 200)
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@Column(name = "description", length = 1000)
	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	@Column(name = "type", length = 200)
	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	// not null?
	@JoinColumn(name = "annotator")
	// @WebMethod(exclude = true)//does not work here
	public Account getAnnotator() {
		return annotator;
	}

	public void setAnnotator(Account annotator) {
		this.annotator = annotator;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	// TODO cascade not necessary?
	@JoinColumn(name = "project")
	public AnnotationProjectDb getAnnotationProject() {
		if (project == null) {
			return null;
		} else {
			if (project.getClass() != AnnotationProjectDb.class) {
				project = new AnnotationProjectDb(project);
			}
			return (AnnotationProjectDb) project;
		}
	}

	public void setAnnotationProject(AnnotationProjectDb anno) {
		this.project = anno;
	}

	/**
	 * Warning: Changes on the returned Set will not affect this Set. Use
	 * setAnnotations after changing!
	 * 
	 * @return a Copy of the contained Set<AnnotationGroup> as
	 *         Set<AnnotationGroupDb>
	 */
	@OneToMany(fetch = FetchType.LAZY, cascade = { CascadeType.REMOVE })
	@JoinTable(name = "annotation_groups_in_session")
	public Set<AnnotationGroupDb> getAnnotations() {
		Set<AnnotationGroupDb> dbAnnotations = new HashSet<AnnotationGroupDb>();
		Iterator<AnnotationGroup> it = this.annotations.iterator();
		while (it.hasNext()) {
			AnnotationGroup next = it.next();
			if (next.getClass() != AnnotationGroupDb.class) {
				next = new AnnotationGroupDb(next);
			}
			dbAnnotations.add((AnnotationGroupDb) next);
		}
		return dbAnnotations;
	}

	public void setAnnotations(Set<AnnotationGroupDb> annotations) {
		this.annotations.clear();
		this.annotations.addAll(annotations);
	}
}
